@@ -30,9 +30,6 @@
 #include "ev-transition-animation.h"
 #include "ev-view-cursor.h"
 #include "ev-page-cache.h"
-#ifdef GDK_WINDOWING_WAYLAND
-#include <gdk/gdkwayland.h>
-#endif
 
 enum {
 	PROP_0,
@@ -263,9 +260,9 @@ ev_view_presentation_transition_start (EvViewPresentation *pview)
 							     pview->current_page);
 	if (duration >= 0) {
 		        pview->trans_timeout_id =
-				g_timeout_add_seconds (duration,
-						       (GSourceFunc) transition_next_page,
-						       pview);
+				g_timeout_add (duration * 1000,
+					       (GSourceFunc) transition_next_page,
+					       pview);
 	}
 }
 
@@ -933,7 +930,7 @@ ev_view_presentation_set_cursor (EvViewPresentation *pview,
 
 	cursor = ev_view_cursor_new (gtk_widget_get_display (widget), view_cursor);
 	gdk_window_set_cursor (gtk_widget_get_window (widget), cursor);
-	gdk_flush ();
+	gdk_display_flush (gtk_widget_get_display (widget));
 	if (cursor)
 		g_object_unref (cursor);
 }
@@ -1290,16 +1287,14 @@ ev_view_presentation_update_monitor_geometry (EvViewPresentation *pview)
 	monitor = gdk_display_get_monitor_at_window (display, window);
 	gdk_monitor_get_geometry (monitor, &geometry);
 
+#if GTK_CHECK_VERSION(3, 24, 9)
+	/* See Evince issue #1365 and GTK regression gtk#2599 */
+	int scale_factor = gdk_monitor_get_scale_factor (monitor);
+	pview->monitor_width = geometry.width / scale_factor;
+	pview->monitor_height = geometry.height / scale_factor;
+#else
 	pview->monitor_width = geometry.width;
 	pview->monitor_height = geometry.height;
-
-#if GTK_CHECK_VERSION(3, 24, 9) && defined (GDK_WINDOWING_WAYLAND)
-	if (GDK_IS_WAYLAND_DISPLAY (display)) {
-		/* See Evince issue #1365 and GTK regression gtk#2599 */
-		int scale_factor = gdk_monitor_get_scale_factor (monitor);
-		pview->monitor_width = geometry.width / scale_factor;
-		pview->monitor_height = geometry.height / scale_factor;
-	}
 #endif
 }
 
